@@ -31,16 +31,14 @@
 #define ST_FORMAT_H
 
 #include "main/formats.h"
-#include "main/mtypes.h"
+#include "main/glheader.h"
 
 #include "pipe/p_defines.h"
 #include "pipe/p_format.h"
 
+struct gl_context;
 struct pipe_screen;
 
-extern GLenum
-st_format_datatype(enum pipe_format format);
-
 
 extern enum pipe_format
 st_mesa_format_to_pipe_format(gl_format mesaFormat);
@@ -51,22 +49,25 @@ st_pipe_format_to_mesa_format(enum pipe_format pipeFormat);
 
 extern enum pipe_format
 st_choose_format(struct pipe_screen *screen, GLenum internalFormat,
+                 GLenum format, GLenum type,
                  enum pipe_texture_target target, unsigned sample_count,
-                 unsigned tex_usage);
+                 unsigned bindings);
 
 extern enum pipe_format
 st_choose_renderbuffer_format(struct pipe_screen *screen,
                               GLenum internalFormat, unsigned sample_count);
 
 
+gl_format
+st_ChooseTextureFormat_renderable(struct gl_context *ctx, GLint internalFormat,
+				  GLenum format, GLenum type, GLboolean renderable);
+
 extern gl_format
-st_ChooseTextureFormat(GLcontext * ctx, GLint internalFormat,
+st_ChooseTextureFormat(struct gl_context * ctx, GLenum target,
+                       GLint internalFormat,
                        GLenum format, GLenum type);
 
 
-extern GLboolean
-st_equal_formats(enum pipe_format pFormat, GLenum format, GLenum type);
-
 /* can we use a sampler view to translate these formats
    only used to make TFP so far */
 extern GLboolean
@@ -77,5 +78,4 @@ extern void
 st_translate_color(const GLfloat colorIn[4], GLenum baseFormat,
                    GLfloat colorOut[4]);
 
-
 #endif /* ST_FORMAT_H */
